#include "rational.hpp"

long int pgcd(long int a,long int b){
  long int rp=b;
  long int r=a;
  long int t;
  while(r!=0){
    t=r;
    r=rp%r;
    rp=t;
  }
  if(rp<0) return -rp;
  return rp;
}

void Rational::reduce(){
  long int d=pgcd(p,q);
  p=p/d;
  q=q/d;
  if(q<0){
    p=-p;
    q=-q;
  }
}

Rational::Rational(){ //Create 0 (i.e. 0/1)
  p=0;
  q=1;
}

Rational::Rational(long int _p){ //Create _p (i.e. _p/1)
  p=_p;
  q=1;
}

Rational::Rational(long int _p,long int _q){ //Create _p/_q
  p=_p;
  q=_q;
  reduce();
}

Rational::Rational(const Rational& r){ //Create a rational from r
  p=r.p;
  q=r.q;
}

Rational& Rational::operator=(const Rational& r){
  p=r.p;
  q=r.q;
  return *this;
}

Rational Rational::operator+(const Rational& r) const{
  Rational res(r.q*p+r.p*q,q*r.q);
  return res;
}

Rational Rational::operator-(const Rational& r) const{
  Rational res(r.q*p-r.p*q,q*r.q);
  return res;
}

Rational Rational::operator-() const{
  Rational res;
  res.p=-p;
  res.q=q;
  return res;
}

Rational Rational::operator*(const Rational& r) const{
  Rational res(p*r.p,q*r.q);
  return res;
}

Rational Rational::operator/(const Rational& r) const{
  Rational res(p*r.q,q*r.p);
  return res;

}

bool Rational::operator==(const Rational& r) const{
  return (p==r.p) and (q==r.q);
  
}

bool Rational::operator!=(const Rational& r) const{
  return (p!=r.p) or (q!=r.q);
}

bool Rational::operator<(const Rational& r) const{
  return p*r.q<r.p*q;
}

bool Rational::operator>(const Rational& r) const{
  return p*r.q>r.p*q;
}

bool Rational::operator<=(const Rational& r) const{
  return p*r.q<=r.p*q;
}

bool Rational::operator>=(const Rational& r) const{
   return p*r.q>=r.p*q;
}

Rational operator+(const long int& a,const Rational& r){
  return Rational(a)+r;
}

Rational operator-(const long int& a,const Rational& r){
  return Rational(a)-r;
}

Rational operator*(const long int& a,const Rational& r){
  return Rational(a)*r;
}

Rational operator/(const long int& a,const Rational& r){
  return Rational(a)/r;
}

bool operator==(const long int& a,const Rational& r){
  return Rational(a)==r;
}

bool operator!=(const long int& a,const Rational& r){
  return Rational(a)!=r;
}

bool operator<(const long int& a,const Rational& r){
  return Rational(a)<r;
}

bool operator>(const long int& a,const Rational& r){
  return Rational(a)>r;
}

bool operator<=(const long int& a,const Rational& r){
  return Rational(a)<=r;
}

bool operator>=(const long int& a,const Rational& r){
  return Rational(a)>=r;
}

ostream& operator<<(ostream& os,const Rational& r){
  if(r.q==1) return os<<r.p;
  return os<<r.p<<"/"<<r.q;
}



